using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace Frief
{

    public class BDSMSeat: MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Frief";
        public string pluginName = "BDSM-detainment chair";
        public string pluginVersion = "1";
        public string pluginDate = "06/11" + "/2021";
        public string pluginDescription = @"Used";
        #endregion

        #region Vars    
     
        protected JSONStorableFloat ScrubA;
        protected JSONStorableFloat ScrubB;
        protected JSONStorableFloat ScrubC;
        protected JSONStorableFloat ScrubD;

        private float previousValue1;
        private float previousValue2;
        private float previousValue3;
        private float previousValue4;

        private float maxY;
        private float minY;
        private float ScrubB_maxY;
        private float ScrubB_minY;
        private float ScrubC_maxY;
        private float ScrubC_minY;
        private float ScrubD_maxY;
        private float ScrubD_minY;

        private float seatX;
        private float seatY;
        private float seatZ;


        private float midY;


        private string objectNameSearchString = "BDSMSeat";
        public Transform ObjectRoot;
        public Transform BDSMarmrest;
        public Transform BDSMSitt;
        public Transform BDSMNeckholder;
        public Transform BDSM_Headcuff;


        private Renderer BDSMarmrestRenderer;


        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 360; //3600 for one minute

        private static Atom currentAtom;
        private static string currentAtomName = "None";
        private static string tempAtomName = "None";

        #endregion

        public override void Init()
        {
            try
            {
                if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("BDSM-Seat: This CUA could not be initialized properly.Please select a compatible object and reload the script.");
                    return;
                }


                //**************  ScrubA
                maxY = 2.044369f;
                minY = 1.046843f;

                ScrubA = new JSONStorableFloat("Armrest", 0.011f, -0.13f, 0.13f, true);
                RegisterFloat(ScrubA);
                CreateSlider(ScrubA, false);

                //**************  ScrubB
                ScrubB_maxY = 0.8910211f;
                ScrubB_minY = -0.5831977f;

                ScrubB = new JSONStorableFloat("Seat", 0.17f, -0.5f, 1.4f, true);
                RegisterFloat(ScrubB);
                CreateSlider(ScrubB, false);

                //**************  ScrubC
                ScrubC_maxY = 0.8910211f;
                ScrubC_minY = -0.5831977f;

                ScrubC = new JSONStorableFloat("NackHolder UP/DOWN", 4.51f, 3.57f, 5.37f, true);
                RegisterFloat(ScrubC);
                CreateSlider(ScrubC, false);

                //**************  ScrubD
                ScrubD_maxY = 0.8910211f;
                ScrubD_minY = -0.5831977f;

                ScrubD = new JSONStorableFloat("NackHolder OPEN/CLOSE", 0f, 0f, 100f, true);
                RegisterFloat(ScrubD);
                CreateSlider(ScrubD, false);


                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("Help",
                    "<color=#000><size=35><b>Help</b></size></color>\n\n" +
                    "<color=#333>" +
                    "Simple script to allow you to adjust the BDSM detainment chair .\n\n" +
                    "Youn can find it under https://hub.virtamate.com/members/frief.1649 \n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 1100.0f;

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        
        {

        

            var Slider1 = ScrubA.val;
            var Slider2 = ScrubB.val;
            var Slider3 = ScrubC.val;
            var Slider4 = ScrubD.val;

            if (initLoops < maxInitLoops && editorInitialized == false)
            {
                initLoops++;
                InitEditor();
            }
            if (initLoops == maxInitLoops)
            {
                //SuperController.LogError("BDSMSaddle : This CUA could not be initialized properly. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }

           


            if (previousValue1 != Slider1)
            {
                Slider1Changed();
            }

            if (previousValue2 != Slider2)
            {
                Slider2Changed();
            }

            if (previousValue3 != Slider3)
            {
                Slider3Changed();
            }

            if (previousValue4 != Slider4)
            {
                Slider4Changed();
            }
        }

        private bool InitEditor()
        {
            if (editorInitialized == true) return true;

            ObjectRoot = getObjectRoot(containingAtom.reParentObject);
            if (ObjectRoot == null)
            {
                return false;
            }

            BDSMSitt = ObjectRoot.Find("Seat");
            BDSMarmrest = ObjectRoot.Find("Arms");
            BDSMNeckholder = ObjectRoot.Find("NackHolder");
            BDSM_Headcuff = ObjectRoot.Find("NackHolder/HeadCuff");



            seatX = BDSMarmrest.transform.localPosition.x;
            seatY = BDSMarmrest.transform.localPosition.y;
            seatZ = BDSMarmrest.transform.localPosition.z;

            return false;
        }

        private Transform getObjectRoot(Transform parent)
        {
            foreach (Transform child in parent)
            {
                if (child.name.Contains(objectNameSearchString))
                {
                    return child;
                }
                else
                {
                    Transform childSearch = getObjectRoot(child);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        void Slider1Changed()
        {

          //SuperController.LogMessage("Pos:" + BDSMarmrest.transform.position.y);
          //SuperController.LogMessage("Slider." + ScrubA.val);

            var Slider1 = ScrubA.val;
            var a = Slider1;

            if (BDSMarmrest.transform.position.y < minY)
            {
                BDSMarmrest.localPosition = new Vector3(seatX, minY, seatZ);
                previousValue1 = Slider1;
            }

            if (BDSMarmrest.transform.position.y > maxY)

            {
                BDSMarmrest.localPosition = new Vector3(seatX, maxY, seatZ);
                previousValue1 = Slider1;
            }

            BDSMarmrest.transform.localPosition = new Vector3(seatX, Slider1, seatZ);
            previousValue1 = Slider1;
        }

        void Slider2Changed()
        {

            //SuperController.LogMessage("Pos:" + BDSMSitt.transform.localPosition.y);
            //SuperController.LogMessage("Slider." + ScrubB.val);

            var ancor_pX = BDSMSitt.transform.localPosition.x;
            var ancor_pY = BDSMSitt.transform.localPosition.y;
            var ancor_pZ = BDSMSitt.transform.localPosition.z;
            var Slider2 = ScrubB.val;
            var a = Slider2;

            if (BDSMSitt.transform.position.y < ScrubB_minY)
            {
                BDSMSitt.localPosition = new Vector3(ancor_pX, ScrubB_minY, ancor_pZ);
                previousValue2 = Slider2;
            }


            if (BDSMSitt.transform.position.y > ScrubB_maxY)

            {
                BDSMSitt.localPosition = new Vector3(ancor_pX, ScrubB_maxY, ancor_pZ);
                previousValue2 = Slider2;
            }

            BDSMSitt.localPosition = new Vector3(ancor_pX, Slider2 / 10, ancor_pZ);
            previousValue2 = Slider2;

        }

        void Slider3Changed()
        {

            //SuperController.LogMessage("Pos:" + BDSMNeckholder.transform.localPosition.y);
            //SuperController.LogMessage("Slider." + ScrubC.val);

            var ancor_pX = BDSMNeckholder.transform.localPosition.x;
            var ancor_pY = BDSMNeckholder.transform.localPosition.y;
            var ancor_pZ = BDSMNeckholder.transform.localPosition.z;
            var Slider3 = ScrubC.val;
            var a = Slider3;

            if (BDSMNeckholder.transform.position.y < ScrubC_minY)
            {
                BDSMNeckholder.localPosition = new Vector3(ancor_pX, ScrubC_minY, ancor_pZ);
                previousValue3 = Slider3;
            }


            if (BDSMNeckholder.transform.position.y > ScrubC_maxY)

            {
                BDSMNeckholder.localPosition = new Vector3(ancor_pX, ScrubC_maxY, ancor_pZ);
                previousValue3 = Slider3;
            }

            BDSMNeckholder.localPosition = new Vector3(ancor_pX, Slider3 / 10, ancor_pZ);
            previousValue3 = Slider3;

        }

        void Slider4Changed()
        {

            SuperController.LogMessage("Pos:" + BDSM_Headcuff.transform.localPosition.y);
            SuperController.LogMessage("Slider." + ScrubD.val);

            var ancor_pX = BDSM_Headcuff.transform.localPosition.x;
            var ancor_pY = BDSM_Headcuff.transform.localPosition.y;
            var ancor_pZ = BDSM_Headcuff.transform.localPosition.z;
            var Slider4 = ScrubD.val;
            var a = Slider4;

            if (BDSM_Headcuff.transform.position.y < ScrubD_minY)
            {
                BDSM_Headcuff.localRotation = Quaternion.Euler(0f, ScrubC_minY, 0f);

                previousValue4 = Slider4;
            }


            if (BDSM_Headcuff.transform.position.y > ScrubD_maxY)

            {
                BDSM_Headcuff.localRotation = Quaternion.Euler(0f, ScrubD_maxY, 0f);

                previousValue4 = Slider4;
            }

            //BDSM_Headcuff.localPosition = new Vector3(ancor_pX, Slider4 / 10, ancor_pZ);
            BDSM_Headcuff.localRotation = Quaternion.Euler(0f, Slider4 /2, 0f);
            previousValue4 = Slider4;

        }
    }
}